
package icfp2001;

import java.util.*;

public class Absorber extends EditorVisitor
{
    public final static int MAX = 10;

    public final static double EXPANDFRAC = 0.1;
    public final static int MAXDEPTH = 4;

    protected boolean editableNode(TagNode pNode)
    {
        return editable(pNode);
    }

    protected Node edit(TagNode pNode)
    {
        return absorbEdit(pNode);
    }

    protected boolean editableNode(RootNode pNode)
    {
        return editable(pNode);
    }

    protected Node edit(RootNode pNode)
    {
        return absorbEdit(pNode);
    }

    protected boolean editable(Node pNode)
    {
        return (pNode.numChildren() > 1);
    }

    protected Node absorbEdit(Node pNode)
    {
//System.out.println("Starting");
//NodeWriter.print(pNode);
//System.out.println();
        int vNumChildren;
        TagNode[][] vMatrix;
        
        TagNode vTagNode;
        Node vNode = pNode.deepCopy();
        vNode.setParent(null);

        if (Math.random() < EXPANDFRAC)
        {
            Expander vExp =  new Expander((int)(Math.random()*Math.random()*MAXDEPTH));

            vNode.accept(vExp);
        }
//System.out.println("Expanded");
//NodeWriter.print(vNode);
//System.out.println();
//System.out.println("filling with: " + pNode.getEntryContext());        
        vNode.setEntryContext(pNode.getEntryContext());
        ContextFiller vFiller = new ContextFiller();
        vFiller.setContext(pNode.getEntryContext());
        vNode.accept(vFiller);
        
//System.out.println("Filled");
//NodeWriter.print(vNode);
//System.out.println();
//vNode.accept(new NodeContextWriter(System.out));
//System.out.println();
        
        collapse(vNode);
//System.out.println("Collapsed");
//NodeWriter.print(vNode);
//System.out.println();

        vNumChildren = vNode.numChildren();
        vMatrix = getAbsorptionMatrix(vNode);

        for (int vAttempts = 0; vAttempts < MAX; vAttempts++)
        {
            boolean vSuccess = false;
            int i=0;
            int j=0;
            int vHits = 0;
            int vTarget = pickRandomNum(vNumChildren*vNumChildren/2);
            while (vHits < vTarget)
            {
                for (int vTestI = 0; vHits < vTarget && vTestI < vNumChildren; vTestI++)
                {
                    for (int vTestJ = vTestI+1; vHits < vTarget && vTestJ < vNumChildren; vTestJ++)
                    {
                        if (vMatrix[vTestI][vTestJ] != null && vMatrix[vTestI][vTestJ].cost() != 0)
                        {
                            vHits++;
                            i=vTestI;
                            j=vTestJ;
                        }
                    }
                }

                if (vHits == 0)
                {
                    return null;
//                    vHits = vTarget;
                }
            }

            if (vMatrix[i][j] != null && vMatrix[i][j].cost() != 0)
            {
                vSuccess = attemptAbsorb(vNode, i, j, vMatrix[i][j]);
            }
            
            if (vSuccess)
            {
                vNumChildren = vNode.numChildren();
                vMatrix = getAbsorptionMatrix(vNode);
            }
        }

        if (Cost.calc(vNode) < Cost.calc(pNode))
        {
//System.out.println("vNode");        
//NodeWriter.print(vNode);
//System.out.println();
//System.out.println("pNode");        
//NodeWriter.print(pNode);
//System.out.println();
            return swapIn(vNode, pNode);
        }
        else
        {
            return null;
        }
    }

    public int pickRandomNum(int pMax)
    {
        return (int)((Math.random()*pMax));
    }

    public TagNode[][] getAbsorptionMatrix(Node pNode)
    {
        int vNumChildren = pNode.numChildren();
        TagNode[][] vMatrix = new TagNode[vNumChildren][vNumChildren];

        for (int i = 0; i < vNumChildren; i++)
        {
            Node vINode = pNode.getChild(i);
            vMatrix[i] = new TagNode[vNumChildren];
            if (vINode instanceof TagNode)
            {
                for (int j = i+1; j < vNumChildren; j++)
                {
                    Node vJNode = pNode.getChild(j);
                    if (vJNode instanceof TagNode)
                    {
                        vMatrix[i][j] = ((TagNode)vINode).intersection((TagNode)vJNode);
                    }
                }
            }
        }

        return vMatrix;
    }

    public boolean attemptAbsorb(Node pParent, int i, int j, TagNode pTag)
    {
        if (checkAbsorbFeasible(pParent, i, j, pTag))
        {
            performAbsorb(pParent, i, j, pTag);
            return true;
        }
        else
        {
            return false;
        }
    }

    public boolean checkAbsorbFeasible(Node pParent, int i, int j, TagNode pTag)
    {
        // check intermediates for ok-ness
        for (int vIndex = i+1; vIndex < j; vIndex++)
        {
            Node vChild = pParent.getChild(vIndex);

            if (vChild instanceof TagNode)
            {
                TagNode vChildTag = (TagNode)vChild;
                TagNode vProposedChild = ((TagNode)vChild.detachedCopy()).subtract(pTag);
                Context vCurrentResultContext = vChild.getEntryContext().applyTags(vChildTag);
                Context vProposedContext = vChild.getEntryContext().applyTags(pTag).applyTags(vProposedChild);

                if (!vCurrentResultContext.equals(vProposedContext))
                {
                    return false;
                }
            }
            else if (vChild instanceof WhiteSpaceNode)
            {
                Context vCurrentResultContext = vChild.getEntryContext();
                Context vProposedContext = vChild.getEntryContext().applyTags(pTag);
                SpaceContext vCurr = new SpaceContext(vCurrentResultContext);
                SpaceContext vProp = new SpaceContext(vProposedContext);
                
                if (!vCurr.equals(vProp))
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        return true;
    }

    public void performAbsorb(Node pParent, int i, int j, TagNode pTag)
    {
        List vChildren = new ArrayList();

        Node vChild=null;
        
        for (int vIndex = i; vIndex <= j; vIndex++)
        {
            vChild = pParent.getChild(vIndex);
            vChildren.add(vChild);
        }

        if (!vChildren.isEmpty() && vChild != null)
        {

            Iterator vIt = vChildren.iterator();
            while (vIt.hasNext())
            {
                ((Node)vIt.next()).detach();
            }

            pParent.insertChild(i, pTag);

            pTag.setEntryContext(vChild.getEntryContext());

            Context vNewContext = pTag.getEntryContext().applyTags(pTag);
            vIt = vChildren.iterator();
            while (vIt.hasNext())
            {
                vChild = (Node)vIt.next();

                if (vChild instanceof TagNode)
                {
                    TagNode vChildTag = (TagNode)vChild;
                    TagNode vProposedChild = ((TagNode)vChild.detachedCopy()).subtract(pTag);
                    Node vSubChild;
                    while (vChildTag.numChildren() > 0)
                    {
                        vSubChild = vChildTag.getChild(0);
                        vSubChild.detach();
                        vProposedChild.addChild(vSubChild);
                    }
                    vProposedChild.setEntryContext(vNewContext);
                    pTag.addChild(vProposedChild);
                }
                else
                {
                    vChild.setEntryContext(vNewContext);
                    pTag.addChild(vChild);
                }
            }

            collapse(pParent);
        }
    }

    public static void collapse(Node pNode)
    {
//System.out.println("A");
//NodeWriter.print(pNode);
//System.out.println();
        TagCollapser vCollapser = new TagCollapser();
        pNode.accept(vCollapser);
//System.out.println("B");
//NodeWriter.print(pNode);
//System.out.println();
        LeftSidePromoter vLPromoter = new LeftSidePromoter();
        pNode.accept(vLPromoter);
//System.out.println("C");
//NodeWriter.print(pNode);
//System.out.println();
        RightSidePromoter vRPromoter = new RightSidePromoter();
        pNode.accept(vRPromoter);
//System.out.println("D");
//NodeWriter.print(pNode);
//System.out.println();
        EmptyTagDeleter vDeleter = new EmptyTagDeleter();
        pNode.accept(vDeleter);
//System.out.println("E");
//NodeWriter.print(pNode);
//System.out.println();
    }

    public Node swapIn(Node pNewNode, Node pOldNode)
    {
        synchronized (pOldNode)
        {
            Node vChild;
            
            while (pOldNode.numChildren() > 0)
            {
                pOldNode.getChild(0).detach();
            }

            while (pNewNode.numChildren() > 0)
            {
                vChild = pNewNode.getChild(0);
                vChild.detach();

                pOldNode.addChild(vChild);
            }

            return pOldNode;
        }
    }

    public void print(TagNode[][] vMatrix)
    {
        for (int i=0; i < vMatrix.length; i++)
        {
            TagNode[] vLine = vMatrix[i];
            for (int j =0 ; j < vLine.length; j++)
            {
                if (vLine[j] == null)
                {
                    System.out.print(vLine[j] + ",");
                }
                else
                {
                    System.out.print(vLine[j].getTagNames() + ",");
                }
            }
            System.out.println();
        }
        System.out.println();
    }
}

