
package icfp2001;

import java.util.*;

public class Expander extends EditorVisitor
{
    protected int mLimit;
    protected int mCurr;

    public Expander(int pLimit)
    {
        mLimit = pLimit;
    }

    public void expand(Node pNode)
    {
        mCurr = 0;
        pNode.accept(this);
    }

    protected boolean editableNode(TagNode pNode)
    {
        return (mCurr < mLimit && pNode.numChildren() > 1);
    }

    protected Node edit(TagNode pNode)
    {
        Node vParent = pNode.getParent();

        if (vParent != null)
        {
            List vChildren = new ArrayList();
            Node vChild;
            int vIndexInParent = vParent.getChildIndex(pNode);
            Context vNodeContext = pNode.getEntryContext();
            synchronized (vParent)
            {
                while (pNode.numChildren() > 1)
                {
                    vChild = pNode.getChild(0);
                    vChild.detach();
                    vChildren.add(vChild);
                }

                Iterator vIt = vChildren.iterator();
                while(vIt.hasNext())
                {
                    vChild = (Node)vIt.next();

                    TagNode vNewTag =  (TagNode)pNode.detachedCopy();
                    vNewTag.addChild(vChild);
                    vNewTag.setEntryContext(vNodeContext);
                    vParent.insertChild(vIndexInParent, vNewTag);
                    vIndexInParent++;
                }
            }
        }
        return vParent;
    }

    protected Node visitChildren(Node pNode)
    {
        Node vNode = null;
        mCurr++;
        if (mCurr < mLimit)
        {
            vNode = (super.visitChildren(pNode));
        }
        mCurr--;
        return vNode;
    }
      
}
