
package icfp2001;

import java.io.*;
import java.util.*;

/**
 * This class outputs the nodes to a PrintStream
 *
 */
public class NodeContextWriter implements NodeVisitor 
{

    protected PrintStream mOut;

    /**
     * Constructs a new node writer
     *
     * @param pWriter the print writer
     */
    public NodeContextWriter(PrintStream pWriter) 
    {
        mOut = pWriter;
    }

    public Object visitTextNode(TextNode pTextNode) 
    {
        mOut.print(pTextNode.getEntryContext());
        mOut.print(pTextNode.getText());
        return null;
    }

    public Object visitWhiteSpaceNode(WhiteSpaceNode pWhiteSpaceNode) 
    {
        Context vContext = (Context)pWhiteSpaceNode.getEntryContext();
        SpaceContext vSpaceContext = new SpaceContext(vContext);
//        mOut.print(vContext);
        mOut.print(vSpaceContext);
        mOut.print(pWhiteSpaceNode.getText());
        return null;
    }

    public Object visitTagNode(TagNode pTagNode)
    {
        visitChildren(pTagNode);
        return null;
    }

    public Object visitRootNode(RootNode pRootNode)
    {
        visitChildren(pRootNode);
        mOut.flush();
        return null;
    }
    

    protected void visitChildren(Node pNode)
    {
        for (int i = 0; i < pNode.numChildren(); i++)
        {
            pNode.getChild(i).accept(this);
        }
    }
}


