package icfp2001;

import java.util.*;

/**
 * This class generates Node objects appropriate to the given parameters
 *
 */
public class NodeFactory {

    /**
     * Generates a new text node from a String
     *
     * @param pString the String contents of the node
     */
    public Node newTextNode(String pString) {
        return new TextNode(pString);
    }

    /**
     * Generates a new white space node from a string
     *
     * @param pString the String of whitespace
     */
    public Node newWhiteSpaceNode(String pString) {
        return new WhiteSpaceNode(pString);
    }

    /**
     * Generates a new tag node
     * 
     * @param pTagType a String representing the tag type
     * @param pChildren a List of children Nodes
     */
    public Node newTagNode(String pTagType, List pChildren) {
        /* probably will create different Node types for different tags.
         * For now, just create a generic tag
         */
        TagNode vTagNode = new TagNode(pTagType);
        vTagNode.addChildren(pChildren);

        return vTagNode;
    }    

    /**
     * Generates a new root node
     * 
     * @param pChildren a List of children Nodes
     */
    public Node newRootNode(List pChildren) {
        RootNode vRootNode = new RootNode();
        vRootNode.addChildren(pChildren);

        return vRootNode;
    }    

}
