
package icfp2001;

import java.io.*;
import java.util.*;

/**
 * This class outputs the nodes to a PrintStream
 *
 */
public class NodeWriter implements NodeVisitor 
{

    public static void print(Node pNode)
    {
        NodeWriter vWriter = new NodeWriter(System.out);
        pNode.accept(vWriter);
    }

    protected PrintStream mOut;

    /**
     * Constructs a new node writer
     *
     * @param pWriter the print writer
     */
    public NodeWriter(PrintStream pWriter) 
    {
        mOut = pWriter;
    }

    public Object visitTextNode(TextNode pTextNode) 
    {
        mOut.print(pTextNode.getText());
        return null;
    }

    public Object visitWhiteSpaceNode(WhiteSpaceNode pWhiteSpaceNode) 
    {
        mOut.print(pWhiteSpaceNode.getText());
        return null;
    }

    public Object visitTagNode(TagNode pTagNode)
    {
        List vTags = pTagNode.getTagNames();
        printOpenTags(vTags);
        visitChildren(pTagNode);
        printCloseTags(vTags);
        return null;
    }

    public void printOpenTags(List vTags)
    {
        Iterator vIt = vTags.iterator();
        while(vIt.hasNext())
        {
            mOut.print("<");
            mOut.print(vIt.next());
            mOut.print(">");
        }
    }

    public void printCloseTags(List vTags)
    {
        for (int i = vTags.size()-1; i >= 0; i--)
        {
            mOut.print("</");
            mOut.print(vTags.get(i));
            mOut.print(">");
        }
    }

    public Object visitRootNode(RootNode pRootNode)
    {
        visitChildren(pRootNode);
        mOut.flush();
        return null;
    }

    protected void visitChildren(Node pNode)
    {
        for (int i = 0; i < pNode.numChildren(); i++)
        {
            pNode.getChild(i).accept(this);
        }
    }
}

