
package icfp2001;


public class TagCollapser extends EditorVisitor
{

    protected boolean editableNode(TagNode pNode)
    {
        if (pNode.numChildren() == 1)
        {
            if (pNode.getChild(0) instanceof TagNode)
            {
                return true;
            }
        }
        return false;
    }
    

    protected Node edit(TagNode pNode)
    {
        TagNode vChild = (TagNode)pNode.getChild(0);
        Context vNodeContext = (Context)pNode.getEntryContext();        

        synchronized (pNode)
        {
//System.out.println("Before pNode = " + pNode.getTagNames());
//NodeWriter.print(pNode);
//System.out.println();
//System.out.println("Before vChild = " + vChild.getTagNames());
//NodeWriter.print(vChild);
//System.out.println();
        
            if (vChild.isBTag())
            {
                if (!pNode.isPLTag())
                {
                    promoteTag(vChild, pNode, TagNode.B);
                }
            }
            
            if (vChild.isEMTag())
            {
                if (pNode.isEMTag())
                {
                    eliminateEMTags(vChild, pNode);
                } 
                else if ((!pNode.isPLTag()) && (!pNode.isSTag()))
                {
                    promoteTag(vChild, pNode, TagNode.EM);
                }
            }

            if (vChild.isITag())
            {
                if(!pNode.isPLTag())
                {
                    promoteTag(vChild, pNode, TagNode.I);
                }
            }

            if (vChild.isPLTag())
            {
                if ((!pNode.isBTag()) &&
                    (!pNode.isEMTag()) &&
                    (!pNode.isITag()) &&
                    (!pNode.isSTag()) &&
                    (!pNode.isTTTag()) &&
                    (!pNode.isUTag()))
                {
                    promoteTag(vChild, pNode, TagNode.PL);
                }
            }

            if (vChild.isSTag())
            {
                if ((!pNode.isEMTag()) &&
                    (!pNode.isPLTag()))
                {
                    promoteTag(vChild, pNode, TagNode.S);
                }
            }

            if (vChild.isTTTag())
            {
                if (!pNode.isPLTag())
                {
                    promoteTag(vChild, pNode, TagNode.TT);
                }
            }

            if (vChild.isUTag())
            {
                if (!pNode.isPLTag() && !pNode.isUTag())
                {
                    promoteTag(vChild, pNode, TagNode.U);
                }
            }

            if (vChild.isSizeTag())
            {
                if (pNode.isSizeTag())
                {
                    if (vChild.getSize() == pNode.getSize())
                    {
                        promoteSizeTag(vChild, pNode);
                    }
                }
                else
                {
                    promoteSizeTag(vChild, pNode);
                }
            }

            if (vChild.isColorTag())
            {
                if (pNode.isColorTag())
                {
                    if (vChild.getColorValue() == pNode.getColorValue())
                    {
                        promoteColorTag(vChild, pNode);
                    }
                }
                else
                {
                    promoteColorTag(vChild, pNode);
                }
            }
        }

//System.out.println("After pNode = " + pNode.getTagNames());
//System.out.println("AFter vChild = " + vChild.getTagNames());
//System.out.println("Before context = " + vChild.getEntryContext());
        vChild.setEntryContext(vNodeContext.applyTags(pNode));
//System.out.println("After context = " + vChild.getEntryContext());

        return pNode;
    }


    public void promoteTag(TagNode pChild, TagNode pParent, Object pTag)
    {
        pChild.deleteTag(pTag);
        pParent.addTag(pTag);
    }

    public void promoteColorTag(TagNode pChild, TagNode pParent)
    {
        promoteTag(pChild, pParent, TagNode.COLOR);
        pParent.colorValue = pChild.colorValue;
    }

    public void promoteSizeTag(TagNode pChild, TagNode pParent)
    {
        promoteTag(pChild, pParent, TagNode.SIZE);
        pParent.sizeValue = pChild.sizeValue;
    }

    public void eliminateEMTags(TagNode pChild, TagNode pParent)
    {
        pChild.deleteTag(TagNode.EM);
        pParent.deleteTag(TagNode.EM);
    }
}

