package icfp2001;

import java.util.*;

/**
 * A Node which represents a Tag
 *
 */
public class TagNode extends Node 
{
//    public final static Object B = new Object();
//    public final static Object EM = new Object();
//    public final static Object I = new Object();
//    public final static Object PL = new Object();
//    public final static Object S = new Object();
//    public final static Object TT = new Object();
//    public final static Object U = new Object();
//    public final static Object SIZE = new Object();
//    public final static Object COLOR = new Object();
    public final static Object B = "BTAG";
    public final static Object EM = "EMTAG";
    public final static Object I = "ITAG";
    public final static Object PL = "PLTAG";
    public final static Object S = "STAG";
    public final static Object TT = "TTTAG";
    public final static Object U = "UTAG";
    public final static Object SIZE = "SIZETAG";
    public final static Object COLOR = "COLORTAG";

    protected HashSet mTags;

    protected int sizeValue = Context.SIZE_DEFAULT;
    protected int colorValue = Context.COLOR_DEFAULT;


    protected TagNode()
    {
        mTags = new HashSet();
    }

    /**
     * Creates a new TagNode
     *
     * @param pTagName the desired type of the new tag node
     */
    public TagNode(String pTagName)
    {
        this();
        if (pTagName.equals("B"))
        {
            mTags.add(B);
        } 
        else if (pTagName.equals("EM"))
        {
            mTags.add(EM);
        }
        else if (pTagName.equals("I"))
        {
            mTags.add(I);
        }
        else if (pTagName.equals("PL"))
        {
            mTags.add(PL);
        }
        else if (pTagName.equals("S"))
        {
            mTags.add(S);
        }
        else if (pTagName.equals("TT"))
        {
            mTags.add(TT);
        }
        else if (pTagName.equals("U"))
        {
            mTags.add(U);
        }
        else if (pTagName.equals("r"))
        {
            mTags.add(COLOR);
            colorValue = Context.COLOR_R;
        }
        else if (pTagName.equals("g"))
        {
            mTags.add(COLOR);
            colorValue = Context.COLOR_G;
        }
        else if (pTagName.equals("b"))
        {
            mTags.add(COLOR);
            colorValue = Context.COLOR_B;
        }
        else if (pTagName.equals("c"))
        {
            mTags.add(COLOR);
            colorValue = Context.COLOR_C;
        }
        else if (pTagName.equals("m"))
        {
            mTags.add(COLOR);
            colorValue = Context.COLOR_M;
        }
        else if (pTagName.equals("y"))
        {
            mTags.add(COLOR);
            colorValue = Context.COLOR_Y;
        }
        else if (pTagName.equals("k"))
        {
            mTags.add(COLOR);
            colorValue = Context.COLOR_K;
        }
        else if (pTagName.equals("w"))
        {
            mTags.add(COLOR);
            colorValue = Context.COLOR_W;
        }
        else
        {
            mTags.add(SIZE);
            // assume it's a number
            sizeValue = Integer.parseInt(pTagName);
        }
    }

    public void addTag(Object pTag)
    {
        mTags.add(pTag);
    }

    public void deleteTag(Object pTag)
    {
        mTags.remove(pTag);
    }

    public Object accept(NodeVisitor pVisitor)
    {
        return pVisitor.visitTagNode(this);
    }


    public boolean isBTag()
    {
        return mTags.contains(B);
    }

    public boolean isITag()
    {
        return mTags.contains(I);
    }

    public boolean isTTTag()
    {
        return mTags.contains(TT);
    }

    public boolean isUTag()
    {
        return mTags.contains(U);
    }

    public boolean isPLTag()
    {
        return mTags.contains(PL);
    }

    public boolean isSTag()
    {
        return mTags.contains(S);
    }

    public boolean isEMTag()
    {
        return mTags.contains(EM);
    }

    public boolean isSizeTag()
    {
        return mTags.contains(SIZE);
    }

    public int getSize()
    {
        return sizeValue;
    }

    public boolean isColorTag()
    {
        return mTags.contains(COLOR);
    }

    public int getColorValue()
    {
        return colorValue;
    }

    /**
     * Returns a list of tag names
     */
    public List getTagNames()
    {
        List vList = new ArrayList();

        if (isBTag())
        {
            vList.add("B");
        }
        if (isITag())
        {
            vList.add("I");
        }
        if (isTTTag())
        {
            vList.add("TT");
        }
        if (isUTag())
        {
            vList.add("U");
        }
        if (isPLTag())
        {
            vList.add("PL");
        }
        if (isSTag())
        {
            vList.add("S");
        }
        if (isEMTag())
        {
            vList.add("EM");
        }
        if (isSizeTag())
        {
            vList.add(""+getSize());
        }
        if (isColorTag())
        {
            switch (colorValue)
            {
                case Context.COLOR_DEFAULT:
                    break;
                case Context.COLOR_R:
                    vList.add("r");
                    break;
                case Context.COLOR_G:
                    vList.add("g");
                    break;
                case Context.COLOR_B:
                    vList.add("b");
                    break;
                case Context.COLOR_C:
                    vList.add("c");
                    break;
                case Context.COLOR_M:
                    vList.add("m");
                    break;
                case Context.COLOR_Y:
                    vList.add("y");
                    break;
                case Context.COLOR_K:
                    vList.add("k");
                    break;
                case Context.COLOR_W:
                    vList.add("w");
                    break;
            }
        }

        return vList;
    }


    public int cost()
    {
        int vTotal = 0;

        vTotal += isBTag() ? 7 : 0;
        vTotal += isITag() ? 7 : 0;
        vTotal += isUTag() ? 7 : 0;
        vTotal += isSTag() ? 7 : 0;
        vTotal += isSizeTag() ? 7 : 0;
        vTotal += isColorTag() ? 7 : 0;
        vTotal += isTTTag() ? 9 : 0;
        vTotal += isPLTag() ? 9 : 0;
        vTotal += isEMTag() ? 9 : 0;

        return vTotal;
    }

    public boolean mergeable(TagNode pNode)
    {
        if (isBTag())
        {
            if (pNode.isPLTag())
            {
                return false;
            }
        }
        if (isEMTag())
        {
            if (pNode.isEMTag() || pNode.isPLTag() || pNode.isSTag())
            {
                return false;
            }
        }
        if (isITag())
        {
            if(pNode.isPLTag())
            {
                return false;
            }
        }
        if (isPLTag())
        {
            if (pNode.isBTag() || pNode.isEMTag() || pNode.isITag() ||
                pNode.isSTag() || pNode.isTTTag() || pNode.isUTag())
            {
                return false;
            }
        }
        if (isSTag())
        {
            if (pNode.isEMTag() || pNode.isPLTag())
            {
                return false;
            }
        }
        if (isTTTag())
        {
            if (pNode.isPLTag())
            {
                return false;
            }
        }
        if (isUTag())
        {
            if (pNode.isPLTag() || pNode.isUTag())
            {
                return false;
            }
        }
        if (isSizeTag())
        {
            if (pNode.isSizeTag())
            {
                if (getSize() != pNode.getSize())
                {
                    return false;
                }
            }
        }
        if (isColorTag())
        {
            if (pNode.isColorTag())
            {
                if (getColorValue() != pNode.getColorValue())
                {
                    return false;
                }
            }
        }

        return true;
    }

    public TagNode intersection(TagNode pOther)
    {
    
        TagNode vNewTagNode = new TagNode();

        Iterator vIt = mTags.iterator();
        Object vTag=null;
        while(vIt.hasNext())
        {
            vTag = vIt.next();

            if (pOther.mTags.contains(vTag))
            {
                if (vTag.equals(COLOR))
                {
                    if (getColorValue() == pOther.getColorValue())
                    {
                        vNewTagNode.addTag(vTag);
                        vNewTagNode.colorValue = getColorValue();
                    }
                }
                else if (vTag.equals(SIZE))
                {
                    if (getSize() == pOther.getSize())
                    {
                        vNewTagNode.addTag(vTag);
                        vNewTagNode.sizeValue = getSize();
                    }
                }
                else
                {
                    vNewTagNode.addTag(vTag);
                }
            }

        }

        return vNewTagNode;
    }

    public TagNode subtract(TagNode pOther)
    {
    
        TagNode vNewTagNode = new TagNode();

        Iterator vIt = mTags.iterator();
        Object vTag=null;
        while(vIt.hasNext())
        {
            vTag = vIt.next();

            if (pOther.mTags.contains(vTag))
            {
                if (vTag.equals(COLOR))
                {
                    if (getColorValue() != pOther.getColorValue())
                    {
                        vNewTagNode.addTag(vTag);
                        vNewTagNode.colorValue = getColorValue();
                    }
                }
                else if (vTag.equals(SIZE))
                {
                    if (getSize() != pOther.getSize())
                    {
                        vNewTagNode.addTag(vTag);
                        vNewTagNode.sizeValue = getSize();
                    }
                }
            }
            else
            {
                vNewTagNode.addTag(vTag);
            }

        }

        return vNewTagNode;
    }

    public Node deepCopy()
    {
        TagNode vNew = (TagNode)super.deepCopy();
        vNew.mTags = (HashSet)mTags.clone();

        return vNew;
    }

    public Node detachedCopy()
    {
        TagNode vNew = (TagNode)super.detachedCopy();
        vNew.mTags = (HashSet)mTags.clone();

        return vNew;
    }

    
}
